/** \file grlib.c
 * \brief <      >
 * <      >
 * \par
 * \author ARV
 * \note <  >
 * \n :
 * \n \date	20.02.2012
 * \par
 * \version <>.	\n
 * Copyright 2012  ARV. All rights reserved.
 * \par
 *   :\n
 * 	-# WinAVR-20080411    
 *
 */

#include <stdint.h>
#include <stdlib.h>
#include <util/delay.h>
#include "font.h"
#include "grlib.h"

static uint8_t	current_page = 0;
static uint16_t	page_addr = 0;
static coord_t cur_x = 0, cur_y = 0;
static coord_t left = 0, right = T6963C_PIXEL_WIDTH-1, top=0, bottom = T6963C_PIXEL_HEIGHT-1;
static draw_mode_t d_mode = DRAW_NORMAL;

draw_mode_t draw_mode(draw_mode_t mode){
	draw_mode_t tmp = d_mode;
	d_mode = mode;
	return tmp;
}

void view_page(uint8_t page){
	if(page >= T6963C_TOTAL_PAGES) return;
	current_page = page;
	set_gr_home_addr(current_page * T6963C_GRAPH_PAGE_SZ);
}

void set_page(uint8_t page){
	if(page >= T6963C_TOTAL_PAGES) return;
	page_addr = page * T6963C_GRAPH_PAGE_SZ;
}

uint8_t active_page(void){
	return page_addr / T6963C_GRAPH_PAGE_SZ;
}

uint8_t viewed_page(void){
	return current_page;
}

void clr_scr(void){
	T6963C_ram_set(page_addr, T6963C_GRAPH_PAGE_SZ, 0);
	fullscreen();
	cur_x = 0;
	cur_y = 0;
}

static uint8_t pt_in_region(coord_t x, coord_t y){
	return (x >= left) && (x <= right) && (y >= top) && (y <= bottom);
}

void pixel(coord_t x, coord_t y){
	static uint16_t adr = 0;

	//      -   
	if(!pt_in_region(x,y)) return;

	//       8    
	div_t result = div(x,8);
	//     
	uint16_t tmp = page_addr + y * T6963C_X_BYTES + result.quot;
	if(tmp != adr){
		//         -   
		adr = tmp;
		set_addr_pointer(adr);
	}
	if(d_mode == DRAW_NORMAL)
		T6963C_write_cmd(CMD_BIT_SET(7 - result.rem));
	else
		T6963C_write_cmd(CMD_BIT_CLR(7 - result.rem));
}

void move_to(coord_t x, coord_t y){
	cur_x = x;
	cur_y = y;
}

#define sign(x) ((x) > 0 ? 1 : (x) < 0 ? -1 : 0)
#define ABS(x)	((x) < 0 ? -(x) : (x))

/**  .
 *         .
 * @param x
 * @param y
 */
void line_to(coord_t x, coord_t y){
	int16_t x1, y1;
	int16_t dx, dy, incx, incy, pdx, pdy, ddx, ddy, es, el, err;

	dx = (x - cur_x);
	dy = (y - cur_y);

	incx = sign(dx);
	incy = sign(dy);

	dx = ABS(dx);
	dy = ABS(dy);

	if (dx > dy){
		pdx = incx;
		pdy = 0;
		ddx = incx;
		ddy = incy;
		es = dy;
		el = dx;
	} else {
		pdx = 0;
		pdy = incy;
		ddx = incx;
		ddy = incy;
		es = dx;
		el = dy;
	}

	x1 = cur_x;
	y1 = cur_y;
	err = el/2;

	pixel(x1,y1);
	for (uint8_t t = 0; t < el; t++){
		err -= es;
		if (err < 0){
			err += el;
			x1 += ddx;
			y1 += ddy;
		} else {
			x1 += pdx;
			y1 += pdy;
		}
		pixel(x1,y1);
	}
	cur_x = x;
	cur_y = y;
}

/**  .
 *    .  TransparentFill != 0,     FillColor
 * @param x1    
 * @param y1  Y  
 * @param r  
 */
void circle(coord_t r){
	int16_t x = 0;
	int16_t y = r;
	int16_t delta = 2 - 2 * r;
	int16_t error = 0;

	while(y >= 0){
		pixel(cur_x + x-1, cur_y + y);
		pixel(cur_x + x-1, cur_y - y);
		pixel(cur_x - x+1, cur_y + y);
		pixel(cur_x - x+1, cur_y - y);

		error = 2 * (delta + y) - 1;
	    if ((delta < 0) && (error < 0)){
	    	delta += 2 * ++x + 1;
	    	continue;
	    }
	    error = 2 * (delta - x) - 1;
	    if ((delta > 0) && (error > 0)){
	    	delta += 1 - 2 * --y;
	    	continue;
	    }
	    x++;
	    delta += 2 * (x - y);
	    y--;
	}
}


void fcircle(coord_t r){				//  
	int16_t x = 0, oldx = -1;
	int16_t y = r;
	int16_t delta = 2 - 2 * r;
	int16_t error = 0;
	coord_t x1 = cur_x, y1 = cur_y;

	while(y >= 0){
		if(x != oldx){
			oldx = x;
			move_to(x1 + x, y1 + y);
			line_to(x1 + x, y1 - y);
			if(x){
				move_to(x1 - x, y1 + y);
				line_to(x1 - x, y1 - y);
			}
		}
		pixel(x1 + x, y1 + y);
		pixel(x1 + x, y1 - y);
		pixel(x1 - x, y1 + y);
		pixel(x1 - x, y1 - y);

		error = 2 * (delta + y) - 1;
	    if ((delta < 0) && (error <= 0)){
	    	delta += 2 * ++x + 1;
	    	continue;
	    }
	    error = 2 * (delta - x) - 1;
	    if ((delta > 0) && (error > 0)){
	    	delta += 1 - 2 * --y;
	    	continue;
	    }
	    x++;
	    delta += 2 * (x - y);
	    y--;
	}
}

void rect(coord_t left, coord_t top, coord_t right, coord_t bottom){
	move_to(left, top);
	line_to(right, top);
	line_to(right, bottom);
	line_to(left, bottom);
	line_to(left, top);
}

void frect(coord_t left, coord_t top, coord_t right, coord_t bottom){
	for(; top <= bottom; top++){
		move_to(left, top);
		line_to(right, top);
	}
}

void set_clip_region(coord_t lt, coord_t tp, coord_t rt, coord_t bt){
	left = lt;
	right = rt;
	top = tp;
	bottom = bt;
}

void draw_vect_pic(const int8_t *ptr){
	coord_t cx = cur_x, cy = cur_y;
	int8_t dx, dy, v;
	while(pgm_read_byte(ptr)){
		v = pgm_read_byte(ptr++);
		dx = pgm_read_byte(ptr++);
		dy = pgm_read_byte(ptr++);
		switch(v){
		case 'M':
			move_to(cx + dx, cy + dy);
			break;
		case 'C':
			move_to(cx + dx, cy + dy);
			circle(pgm_read_byte(ptr++));
			break;
		case 'L':
			line_to(cur_x + dx, cur_y + dy);
			break;
		case 'O':
			move_to(cur_x + dx, cur_y + dy);
			break;
		case 'P':
			pixel(cur_x + dx, cur_y + dy);
			break;
		}
	}
}


void draw_image_P(const image_t *img){
	uint16_t addr;
	const uint8_t *ptr = (void*)&img->bitmap;
	uint8_t w = pgm_read_byte(&img->width)/8, cy = cur_y;
	for(uint8_t y = pgm_read_byte(&img->height); y; y--){
		addr = page_addr + cy++ * T6963C_X_BYTES + cur_x/8;
		ptr = (void*)T6963C_ram_cpy_P(addr, w, ptr);
	}
}

void print_char(char ch){
	uint8_t x = cur_x, y = cur_y;
	if(ch >= FIRST_CHAR){
		for(uint8_t dx=0; dx<SYMBOL_W; dx++){
			uint8_t line = pgm_read_byte(&font[ch-FIRST_CHAR][dx]);
			for(uint8_t mask=0x01, dy=0; mask; mask <<= 1, dy++){
				if(line & mask)
					pixel(x+dx, y+dy);
			}
		}
	}
	move_to(x + SYMBOL_W, y);
}

void print(char *str){
	while(*str){
		print_char(*str++);
	}
}

void print_P(const char *str){
	char ch;
	while(ch = pgm_read_byte(str++)){
		print_char(ch);
	}
}
