/*
	main.c - Sample File

	Simple LowLevel T6963c Library without Drawing functions
	for Rect's, Circles, etc...
	The files should be self-explanatory.

	(c) Simon Kueppers. If you find a bug or if you have a 
	question, feel free to Email me:
	simon.kueppers@stlcd.de
*/

#include "T6963c.h"
#include "grlib.h"
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "images.h"
#include "menu.h"
#include "font.h"

#define R 10

static uint8_t page = 1;

typedef enum{
	ZAZ_COVER	= _BV(0),
	ZAZ_SMOKE	= _BV(1),
	ZAZ_BAG		= _BV(2),
	ZAZ_RUN		= _BV(3),
	ZAZ_CAPOT	= _BV(4),
	ZAZ_LIGHT	= _BV(5),
	ZAZ_DOOR	= _BV(6)
} zaz_t;

void draw_zaz(uint8_t x, uint8_t y, zaz_t mode){

	move_to(x,y); //20
	draw_image_P(&zaz_cabrio);
	if((mode & ZAZ_COVER) && !(mode & ZAZ_RUN)){
		move_to(x+8*5,y-1);
		draw_image_P(&zaz_cover);
	}
	if(mode & ZAZ_SMOKE){
		move_to(x,y+41);
		draw_image_P(&zaz_smoke);
	}
	if(mode & ZAZ_BAG){
		move_to(x,y+23);
		draw_image_P(&zaz_bag);
	}
	if(mode & ZAZ_LIGHT){
		move_to(x+144,y+30);
		draw_image_P(&zaz_fara);
	}
	if(mode & ZAZ_CAPOT){
		move_to(x+120,y-1);
		draw_image_P(&zaz_capot);
	}
	if((mode & ZAZ_RUN) && !(mode & ZAZ_COVER)){
		move_to(x+8,y-3);
		draw_image_P(&zaz_open);
	}
	if(mode & ZAZ_DOOR){
		move_to(x+40,y+3);
		draw_image_P(&zaz_door);
	}
}

#define LIST_CNT	7
static char *list[LIST_CNT] = {
	"String 1",
	"String 2",
	"String 3",
	"String 4",
	"String 5",
	"String 6",
	"String 7"
};

typedef struct{
	uint8_t id;
	int16_t	var;
	void	*func;
	char	*name;
} item_t;

PROGMEM char s1[] = "progmem 1";
PROGMEM char s2[] = "pro demo 3";
PROGMEM char s3[] = "info";
PROGMEM char s4[] = "configuration";
PROGMEM char s5[] = "extra run";
PROGMEM char s6[] = "value";
PROGMEM char s7[] = "prosto 20";

PROGMEM item_t menu[LIST_CNT] = {
	{.id = 1, .var = 25, &list, s1},
	{.id = 2, .var = 255, &list, s2},
	{.id = 2, .var = 255, &list, s3},
	{.id = 2, .var = 255, &list, s4},
	{.id = 2, .var = 255, &list, s5},
	{.id = 2, .var = 255, &list, s6},
	{.id = 2, .var = 255, &list, s7}
};

void printlist(uint8_t item){
	rect(9,0,110, LIST_CNT * (SYMBOL_H + 1)+1);
	for(uint8_t x=10, y=1, i=0; i<LIST_CNT; y+=SYMBOL_H+1, i++){
		text_row(x,y,100,i == item ? ATTR_SELECTED | ALIGN_CT : 0 | ALIGN_CT, pgm_read_word(&menu[i].name));
	}
}

static void ani(void){
	static coord_t i = 0;
	static uint8_t cnt = 0;

	char str[50];

	view_page(page);
	page ^= 1;
	set_page(page);
	clr_scr();
	move_to(i--,0);
	sprintf(str,"  !");
	print(str);
	draw_zaz(0,60,cnt++);
	//printlist(i++);
	//if(i >= LIST_CNT) i = 0;
	//T6963C_ram_xor(T6963C_GRAPH_PAGE_SZ*page, T6963C_GRAPH_PAGE_SZ, 0xF0);
}

int main(){
	T6963C_init();

	while(1){
		ani();
		_delay_ms(1);
	}

}
